<?php

namespace Drupal\access_policy;

use Drupal\access_policy\Entity\AccessPolicyInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Access controller for content entities assigned an AccessPolicy.
 */
class AccessPolicyEntityAccessControlHandler implements ContainerInjectionInterface {

  /**
   * The content access policy manager service.
   *
   * @var \Drupal\access_policy\ContentAccessPolicyManager
   */
  protected $contentAccessPolicyManager;

  /**
   * The AccessRuleValidator service.
   *
   * @var \Drupal\access_policy\AccessRuleValidator
   */
  protected $accessRuleValidator;

  /**
   * Access policy information.
   *
   * @var \Drupal\access_policy\AccessPolicyInformation
   */
  protected $accessPolicyInformation;

  /**
   * Constructs a AccessPolicyEntityAccessControlHandler object.
   *
   * @param \Drupal\access_policy\ContentAccessPolicyManager $content_access_policy_manager
   *   The ContentAccessPolicyManager service.
   * @param \Drupal\access_policy\AccessRuleValidator $access_rule_validator
   *   The AccessRuleValidator service.
   * @param \Drupal\access_policy\AccessPolicyInformation $policy_information
   *   Access policy information.
   */
  public function __construct(ContentAccessPolicyManager $content_access_policy_manager, AccessRuleValidator $access_rule_validator, AccessPolicyInformation $policy_information) {
    $this->contentAccessPolicyManager = $content_access_policy_manager;
    $this->accessRuleValidator = $access_rule_validator;
    $this->accessPolicyInformation = $policy_information;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('access_policy.content_policy_manager'),
      $container->get('access_policy.rule_validator'),
      $container->get('access_policy.information'),
    );
  }

  /**
   * Check access qualifications.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The content entity.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   * @param string $operation
   *   The current operation.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function access(EntityInterface $entity, AccountInterface $account, string $operation) {
    if (!$this->accessPolicyInformation->hasEnabledForEntitiesOfEntityType($entity->getEntityType(), $entity->bundle())) {
      return AccessResult::neutral();
    }
    $access_policy = $this->contentAccessPolicyManager->getAccessPolicy($entity);

    if (!empty($access_policy)) {
      $type = $access_policy->getTypePlugin();

      if ($type->shouldValidatePermission($operation)) {
        $permission = $this->getAccessPolicyPermission($access_policy, $entity->getEntityType(), $operation);
        if (!$account->hasPermission($permission)) {
          return AccessResult::forbidden();
        }
      }

      if ($type->shouldValidateRules($operation)) {
        if (!$account->hasPermission('bypass ' . $access_policy->id() . ' access rules')) {
          $access_rules_check = $this->accessRuleValidator->validateAccessRules($access_policy, $entity, $account, $operation);
          if (!$access_rules_check) {
            return AccessResult::forbidden();
          }
        }
      }

      if ($type->shouldValidatePermission('view unpublished')) {
        if ($this->isViewingUnpublishedEntity($entity, $operation)) {
          $permission = $this->getViewUnpublishedPermission($access_policy, $entity->getEntityType());
          if ($account->hasPermission($permission)) {
            return AccessResult::allowed();
          }
          else {
            return AccessResult::forbidden();
          }
        }
      }
    }

    return $this->accessResult($entity, $operation);
  }

  /**
   * Final access result based on operation.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   * @param string $operation
   *   The operation.
   *
   * @return \Drupal\Core\Access\AccessResultAllowed|\Drupal\Core\Access\AccessResultNeutral
   */
  private function accessResult(EntityInterface $entity, $operation) {
    // The assign operation requires an allowed result otherwise it will return
    // forbidden. All other operations work with neutral.
    if ($operation == 'assign') {
      return AccessResult::allowed()->addCacheableDependency($entity);
    }

    return AccessResult::neutral()->addCacheableDependency($entity);
  }

  /**
   * Determine whether the user is viewing an unpublished entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param string $operation
   *   The operation.
   *
   * @return bool
   *   TRUE if viewing unpublished entity; FALSE otherwise.
   */
  public function isViewingUnpublishedEntity(EntityInterface $entity, $operation) {
    if ($operation == 'view' && $entity instanceof EntityPublishedInterface && !$entity->isPublished()) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Get the view unpublished content permission.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $access_policy
   *   The access policy entity.
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type.
   *
   * @return string
   *   The view unpublished permission.
   */
  private function getViewUnpublishedPermission(AccessPolicyInterface $access_policy, EntityTypeInterface $entity_type) {
    $entity_type_label = strtolower($entity_type->getLabel());
    return 'view any ' . $access_policy->id() . ' unpublished ' . $entity_type_label;
  }

  /**
   * Get the properly formed access policy permission.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $access_policy
   *   The access policy entity.
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type.
   * @param string $op
   *   The operation.
   *
   * @return string
   *   The permission string.
   */
  private function getAccessPolicyPermission(AccessPolicyInterface $access_policy, EntityTypeInterface $entity_type, $op) {
    $entity_type_label = strtolower($entity_type->getLabel());
    switch ($op) {
      case 'update':
        return 'edit ' . $access_policy->id() . ' ' . $entity_type_label;

      case 'assign':
        return 'assign ' . $access_policy->id() . ' access policy';

      case 'view all revisions':
        return 'view all ' . $access_policy->id() . ' ' . $entity_type_label . ' revisions';

      default:
        return $op . ' ' . $access_policy->id() . ' ' . $entity_type_label;
    }
  }

}
