<?php

namespace Drupal\access_policy\Plugin\AccessRuleWidget;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\FormStateInterface;

/**
 * Embedded entity field access rule widget.
 *
 * Renders  the entity field on the access tab.
 *
 * @AccessRuleWidget(
 *   id = "entity_field",
 *   label = @Translation("Entity field"),
 *   description = @Translation("Shows the field on the access tab.")
 * )
 */
class EntityField extends AccessRuleWidgetBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    // Get all the field widget settings for this field type.
    $field_name = $this->getFieldName();
    if (empty($field_name)) {
      return $form;
    }

    // Get the widget options based on field type.
    $field_type = $this->getFieldType($field_name);
    $widgetPluginManager = \Drupal::service('plugin.manager.field.widget');
    $options = $widgetPluginManager->getOptions($field_type);

    $form['field_widget'] = [
      '#title' => $this->t('Field widget display'),
      '#type' => 'select',
      '#default_value' => $this->getWidgetSetting('field_widget'),
      '#options' => $options,
    ];
    return $form;
  }

  /**
   * Get the field name from the plugin.
   *
   * @return string|void
   *   The source field name or NULL if not configured.
   */
  private function getFieldName() {
    // Get all the field widget settings for this field type.
    $plugin = $this->getAccessRulePlugin();
    $definition = $plugin->getPluginDefinition();
    if (isset($definition['field_name'])) {
      return $definition['field_name'];
    }

    // @todo reevaluate field_name and entity_field.
    $config = $plugin->getConfiguration();
    if (!empty($config['entity_field'])) {
      return $config['entity_field'];
    }
  }

  /**
   * Get the field type from the field name.
   *
   * @param string $field_name
   *   The field name.
   *
   * @return string
   *   The field type.
   */
  private function getFieldType($field_name) {
    $policy = $this->getAccessRulePlugin()->getAccessPolicy();
    $entityFieldManager = \Drupal::service('entity_field.manager');

    $field_map = $entityFieldManager->getFieldMap();
    $entity_type = $policy->getTargetEntityType();

    return $field_map[$entity_type->id()][$field_name]['type'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm($form, $form_state) {
    $field_name = $this->getFieldName();
    $entity = $this->getEntity();

    $new_form_display = EntityFormDisplay::create([
      'targetEntityType' => $entity->getEntityTypeId(),
      'bundle' => $entity->bundle(),
      'mode' => 'default',
    ]);

    $options = [];
    if (!empty($this->getWidgetSetting('field_widget'))) {
      $options['type'] = $this->getWidgetSetting('field_widget');
    }
    $new_form_display->setComponent($field_name, $options);

    $widget = $new_form_display->getRenderer($field_name);
    $items = $entity->get($field_name);
    $items->filterEmptyItems();

    $new_form_display->buildForm($entity, $form, $form_state);
    return $widget->form($items, $form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $field_name = $this->getFieldName();
    $value = $form_state->getValue($field_name);

    if (is_array($value)) {
      // This was probably a multi-value field, remove any non-numeric keys.
      foreach ($value as $key => $element) {
        if (!is_numeric($key)) {
          unset($value[$key]);
        }
      }
    }

    $entity = $this->getEntity();
    $entity->set($field_name, $value);
    $entity->save();
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // @todo Implement validateForm() method.
  }

}
